"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.checkPropsObjectTypeMatchesSchema = checkPropsObjectTypeMatchesSchema;
exports.toGraphQLQueryString = toGraphQLQueryString;

var _graphql = require("graphql");

var _flow = require("./flow");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function checkPropsObjectTypeMatchesSchema(schema, fragmentType, objectType, flowTypes) {
  var graphqlType = schema.getType(fragmentType);
  if (!graphqlType || !(graphqlType instanceof _graphql.GraphQLObjectType)) {
    throw new Error("There is no GraphQL object type named '" + fragmentType + "'");
  }

  var flowType = (0, _flow.convertTypeAnnotationToFlowType)(objectType.value, objectType.optional, flowTypes);
  var graphqlTypes = convertGraphqlTypeToFlowType(graphqlType, false);

  var result = compareFlowTypes(flowType, graphqlTypes);
  if (result.length > 0) {
    var errors = result.map(function (r) {
      return "Expected type '" + r.expected + "', actual type '" + r.actual + "' for path " + r.path.join(".");
    });
    throw new Error("Errors for fragment " + fragmentType + ":\n" + errors.join("\n"));
  }
}

function convertGraphqlObjectType(objectType, objectTypeCache) {
  var objectTypeString = objectType.toString();
  if (!objectTypeCache[objectTypeString]) {
    var fields = objectType.getFields();
    // placeholder while we have possible recursive structures
    objectTypeCache[objectTypeString] = true; // eslint-disable-line no-param-reassign
    objectTypeCache[objectTypeString] = Object.keys(fields).reduce(function (obj, key) {
      // eslint-disable-line no-param-reassign
      var field = fields[key];
      return _extends({}, obj, _defineProperty({}, key, convertGraphqlTypeToFlowType(field.type, true, objectTypeCache)));
    }, {});
  }

  return objectTypeCache[objectTypeString];
}

function convertGraphqlTypeToFlowType(graphqlType, nullable) {
  var objectTypeCache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  if (graphqlType instanceof _graphql.GraphQLNonNull) {
    return convertGraphqlTypeToFlowType(graphqlType.ofType, false, objectTypeCache);
  }

  if (graphqlType instanceof _graphql.GraphQLObjectType) {
    return {
      type: "object",
      nullable: nullable,
      properties: convertGraphqlObjectType(graphqlType, objectTypeCache)
    };
  }

  if (graphqlType instanceof _graphql.GraphQLList) {
    return {
      type: "array",
      nullable: nullable,
      child: convertGraphqlTypeToFlowType(graphqlType.ofType, true, objectTypeCache)
    };
  }

  var flowTypeName = graphqlType instanceof _graphql.GraphQLScalarType ? graphqlType.name : "any";
  switch (flowTypeName) {
    case "Boolean":
      return { type: "boolean", nullable: nullable };
    case "Int":
    case "Float":
      return { type: "number", nullable: nullable };
    case "ID":
    case "String":
      return { type: "string", nullable: nullable };
    default:
      return { type: "any", nullable: nullable };
  }
}

var typeToString = function typeToString(type) {
  return (type.nullable ? "?" : "") + type.type;
};

function compareFlowTypes(a, b) {
  var path = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  var actual = typeToString(a);
  var expected = typeToString(b);

  if (actual !== expected) {
    return [{ path: path, actual: actual, expected: expected }];
  }

  // help flow, cant handle an &&
  var aProps = null;
  if (a.type === "object") {
    aProps = a.properties;
  }

  var bProps = null;
  if (b.type === "object") {
    bProps = b.properties;
  }

  if (aProps && bProps) {
    var createPath = function createPath(key) {
      var pathCopy = path.slice();
      pathCopy.push(key);
      return pathCopy;
    };

    var aKeys = Object.keys(aProps).sort();
    return aKeys.reduce(function (result, key) {
      // babel relay plugin handles missing fields
      var bType = bProps && bProps[key];
      if (bType) {
        var aType = aProps && aProps[key];
        if (aType) {
          return result.concat(compareFlowTypes(aType, bType, createPath(key)));
        }
      }
      return result;
    }, []);
  }

  if (a.type === "array" && b.type === "array") {
    return compareFlowTypes(a.child, b.child, path);
  }

  return [];
}

function toGraphQLQueryString(fragmentName, fragmentType, fragmentDirectives, objectType, componentContainerFragments, childFragmentTransformations, flowTypes) {
  var fragmentKey = objectType.key.name;
  var flowType = (0, _flow.convertTypeAnnotationToFlowType)(objectType.value, objectType.optional, flowTypes);

  // remove the first and last lines if flowType is an object as opening braces are added later
  var graphQlQueryBody = flowTypeToGraphQLString(flowType).trim();
  if (graphQlQueryBody) {
    if (graphQlQueryBody[0] === "{") {
      graphQlQueryBody = graphQlQueryBody.substr(graphQlQueryBody.indexOf("\n") + 1);
    }

    if (graphQlQueryBody[graphQlQueryBody.length - 1] === "}") {
      graphQlQueryBody = graphQlQueryBody.substr(0, graphQlQueryBody.lastIndexOf("\n"));
    }
  }

  var childContainersForFragment = Object.keys(componentContainerFragments).reduce(function (c, name) {
    var fragmentKeys = componentContainerFragments[name];
    if (fragmentKeys.indexOf(fragmentKey) >= 0) {
      c.push(name);
    }
    return c;
  }, []);

  var directives = directivesToGraphQLString(fragmentDirectives);
  directives = directives ? directives + " " : "";
  var fragName = fragmentName ? " " + fragmentName : "";
  var graphQlQuery = "\nfragment" + fragName + " on " + fragmentType + " " + directives + "{\n" + graphQlQueryBody;
  var graphQlQueryEnd = "\n}\n";

  var insideChildFragmentStrings = childContainersForFragment.map(function (name) {
    return childFragmentTransformations.insideFragment(name, fragmentKey);
  }).filter(function (fragStr) {
    return fragStr !== null;
  });

  if (insideChildFragmentStrings.length > 0) {
    if (graphQlQueryBody) {
      graphQlQuery += "\n  ";
    }

    graphQlQuery += insideChildFragmentStrings.join("\n  ");
  }

  graphQlQuery += graphQlQueryEnd;

  var outsideChildFragmentStrings = childContainersForFragment.map(function (name) {
    return childFragmentTransformations.outsideFragment(name, fragmentKey);
  }).filter(function (fragStr) {
    return fragStr !== null;
  });

  if (outsideChildFragmentStrings.length > 0) {
    graphQlQuery += outsideChildFragmentStrings.join("\n");
    graphQlQuery += "\n";
  }

  return "" + graphQlQuery;
}

function flowTypeToGraphQLString(flowType) {
  var level = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

  if (flowType.type === "object") {
    var indentation = "  ".repeat(level);

    var props = flowType.properties;
    var strings = Object.keys(props).reduce(function (parts, key) {
      var value = props[key];
      parts.push("" + indentation + key + flowTypeToGraphQLString(value, level + 1));
      return parts;
    }, []);

    return " {\n" + strings.join("\n") + "\n" + "  ".repeat(level - 1) + "}";
  } else if (flowType.type === "array") {
    return flowTypeToGraphQLString(flowType.child, level);
  }

  return "";
}

function directivesToGraphQLString(directives) {
  var directiveNames = Object.keys(directives);
  directiveNames.sort();
  return directiveNames.map(function (directiveName) {
    var directiveArgsMap = directives[directiveName];
    var directiveArgs = Object.keys(directiveArgsMap);
    directiveArgs.sort();

    var directiveArgsStr = directiveArgs.map(function (directiveArg) {
      var directiveArgValue = directiveArgsMap[directiveArg];
      return directiveArg + ": " + JSON.stringify(directiveArgValue);
    }).join(", ");

    return "@" + directiveName + "(" + directiveArgsStr + ")";
  }).join(" ");
}